package com.logicbig.example;


import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import java.util.Arrays;
import java.util.List;

public class Util {

    public static <T> T readEntity(Class<T> entityClass, Object identity, EntityManagerFactory emf) {
        System.out.printf("-- Loading entity %s%n", entityClass.getSimpleName());
        EntityManager em = emf.createEntityManager();
        T t = em.find(entityClass, identity);
        System.out.printf("-- Entity loaded: %s%n", toObjectStr(t));
        return t;
    }

    public static void persistEntity(Object entity, EntityManagerFactory emf) throws Exception {
        EntityManager em = emf.createEntityManager();
        em.getTransaction().begin();
        System.out.println("-- Persisting " + entity.getClass().getSimpleName());
        em.persist(entity);
        System.out.println("-- Entity persisted");
        em.getTransaction().commit();
        System.out.println("-- after committing");
    }

    public static void printCallChain(Object o) {
        List<StackTraceElement> st = Arrays.asList(Thread.currentThread().getStackTrace());
        String callChain = "";
        for (int i = 2; i < st.size(); i++) {
            callChain += getStackTraceElementString(st.get(i)) + "<";
        }
        System.out.printf("Instance: %s, call-stack: %s%n", toObjectStr(o), callChain);
    }

    private static String getStackTraceElementString(StackTraceElement ste) {
        return String.format("%s.%s[%s]", getSimpleClassName(ste.getClassName()),
                ste.getMethodName(), ste.getLineNumber());
    }

    private static String getSimpleClassName(String className) {
        int i = className.lastIndexOf(".");
        if (i != -1) {
            return className.substring(i + 1);
        }
        return className;
    }

    public static String toObjectStr(Object object) {
        return object.getClass().getSimpleName() + "@" + System.identityHashCode(object);
    }
}